! =================================================================================================================================
! MODULE       : stomate_soilcarbon
!
! CONTACT      : orchidee-help _at_ listes.ipsl.fr
!
! LICENCE      : IPSL (2006)
! This software is governed by the CeCILL licence see ORCHIDEE/ORCHIDEE_CeCILL.LIC
!
!>\BRIEF       Calculate soil dynamics largely following the Century model
!!	
!!\n DESCRIPTION: None
!!
!! RECENT CHANGE(S): None
!!
!! SVN		:
!! $HeadURL$ 
!! $Date$
!! $Revision$
!! \n
!_ ================================================================================================================================

MODULE stomate_soilcarbon

  ! modules used:

  USE ioipsl_para
  USE stomate_data
  USE constantes
  USE xios_orchidee
  USE hydrol
  USE vertical_soil_var
  USE constantes_var
  USE pft_parameters
  USE pft_parameters_var
  USE time, ONLY : month_start, one_year, sec_start, julian_diff, one_hour

  IMPLICIT NONE

  ! private & public routines

  PRIVATE
  PUBLIC soilcarbon,soilcarbon_clear

  ! Variables shared by all subroutines in this module
  
  LOGICAL, SAVE    :: firstcall_soilcarbon = .TRUE.   !! Is this the first call? (true/false)
!$OMP THREADPRIVATE(firstcall_soilcarbon)

CONTAINS


!! ================================================================================================================================
!!  SUBROUTINE   : soilcarbon_clear
!!
!>\BRIEF        Set the flag ::firstcall_soilcarbon to .TRUE. and as such activate sections 1.1.2 and 1.2 of the subroutine soilcarbon 
!! (see below).
!! 
!_ ================================================================================================================================
  
  SUBROUTINE soilcarbon_clear
    firstcall_soilcarbon=.TRUE.
  ENDSUBROUTINE soilcarbon_clear


!! ================================================================================================================================
!!  SUBROUTINE   : soilcarbon
!!
!>\BRIEF        Computes the soil respiration and carbon stocks, essentially 
!! following Parton et al. (1987).
!!
!! DESCRIPTION	: The soil is divided into 3 carbon pools, with different 
!! characteristic turnover times : active (1-5 years), slow (20-40 years) 
!! and passive (200-1500 years).\n
!! There are three types of carbon transferred in the soil:\n
!! - carbon input in active and slow pools from litter decomposition,\n
!! - carbon fluxes between the three pools,\n
!! - carbon losses from the pools to the atmosphere, i.e., soil respiration.\n
!!
!! The subroutine performs the following tasks:\n
!!
!! Section 1.\n
!! The flux fractions (f) between carbon pools are defined based on Parton et 
!! al. (1987). The fractions are constants, except for the flux fraction from
!! the active pool to the slow pool, which depends on the clay content,\n
!! \latexonly
!! \input{soilcarbon_eq1.tex}
!! \endlatexonly\n
!! In addition, to each pool is assigned a constant turnover time.\n
!!
!! Section 2.\n
!! The carbon input, calculated in the stomate_litter module, is added to the 
!! carbon stock of the different pools.\n
!!
!! Section 3.\n
!! First, the outgoing carbon flux of each pool is calculated. It is 
!! proportional to the product of the carbon stock and the ratio between the 
!! iteration time step and the residence time:\n
!! \latexonly
!! \input{soilcarbon_eq2.tex}
!! \endlatexonly
!! ,\n
!! Note that in the case of crops, the additional multiplicative factor 
!! integrates the faster decomposition due to tillage (following Gervois et 
!! al. (2008)).
!! In addition, the flux from the active pool depends on the clay content:\n
!! \latexonly
!! \input{soilcarbon_eq3.tex}
!! \endlatexonly
!! ,\n
!! Each pool is then cut from the carbon amount corresponding to each outgoing
!! flux:\n
!! \latexonly
!! \input{soilcarbon_eq4.tex}
!! \endlatexonly\n
!! Second, the flux fractions lost to the atmosphere is calculated in each pool
!! by subtracting from 1 the pool-to-pool flux fractions. The soil respiration 
!! is then the summed contribution of all the pools,\n
!! \latexonly
!! \input{soilcarbon_eq5.tex}
!! \endlatexonly\n
!! Finally, each carbon pool accumulates the contribution of the other pools:
!! \latexonly
!! \input{soilcarbon_eq6.tex}
!! \endlatexonly
!!
!! Section 4.\n
!! If the flag SPINUP_ANALYTIC is set to true, the matrix A is updated following
!! Lardy (2011).
!!
!! RECENT CHANGE(S): None
!! 
!! MAIN OUTPUTS VARIABLE(S): carbon, resp_hetero_soil
!!
!! REFERENCE(S)   :
!! - Parton, W.J., D.S. Schimel, C.V. Cole, and D.S. Ojima. 1987. Analysis of 
!! factors controlling soil organic matter levels in Great Plains grasslands. 
!! Soil Sci. Soc. Am. J., 51, 1173-1179.
!! - Gervois, S., P. Ciais, N. de Noblet-Ducoudre, N. Brisson, N. Vuichard, 
!! and N. Viovy (2008), Carbon and water balance of European croplands 
!! throughout the 20th century, Global Biogeochem. Cycles, 22, GB2022, 
!! doi:10.1029/2007GB003018.
!! - Lardy, R, et al., A new method to determine soil organic carbon equilibrium,
!! Environmental Modelling & Software (2011), doi:10.1016|j.envsoft.2011.05.016
!!
!! FLOWCHART    :
!! \latexonly
!! \includegraphics[scale=0.5]{soilcarbon_flowchart.jpg}
!! \endlatexonly
!! \n
!_ ================================================================================================================================

  SUBROUTINE soilcarbon (npts, clay, &
       soilcarbon_input, control_temp, control_moist, veget_cov_max, &
       mcl_layh, pb, bulk, soil_pH, atmcos, land_mcs, stempdiag, resp_hetero_litter, &
       resp_maint_part_radia, carbon, resp_hetero_soil, MatrixA, nsoil_map)

!! 0. Variable and parameter declaration

    !! 0.1 Input variables
    
    INTEGER(i_std), INTENT(in)                            :: npts             !! Domain size (unitless)
    REAL(r_std), DIMENSION(npts), INTENT(in)              :: nsoil_map         !! Dominant soil type between oxic and anoxic (0-4, unitless)
    REAL(r_std), DIMENSION(npts), INTENT(in)              :: clay             !! Clay fraction (unitless, 0-1) 
    REAL(r_std), DIMENSION(npts,ncarb,nvm), INTENT(in)    :: soilcarbon_input !! Amount of carbon going into the carbon pools from litter decomposition \f$(gC m^{-2} day^{-1})$\f
    REAL(r_std), DIMENSION(npts,nlevs), INTENT(in)        :: control_temp     !! Temperature control of heterotrophic respiration (unitless: 0->1)
    REAL(r_std), DIMENSION(npts,nlevs), INTENT(in)        :: control_moist    !! Moisture control of heterotrophic respiration (unitless: 0.25->1)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)          :: veget_cov_max    !! Fractional coverage: maximum share of the pixel taken by a pft 
    REAL(r_std),DIMENSION (npts,nslm), INTENT(in)         :: mcl_layh         !! Volumetric moisture content for each layer in hydrol (liquid) (m3/m3) 
    REAL(r_std),DIMENSION (npts), INTENT (in)             :: pb               !! Surface pressure (hPa)
    REAL(r_std),DIMENSION(npts),INTENT(in)                :: bulk             !! Bulk density (kg/m**3)
    REAL(r_std),DIMENSION (npts), INTENT (in)             :: soil_pH          !! Soil pH
    REAL(r_std),DIMENSION (:,:), INTENT (in)              :: atmcos           !! Atmospheric COS concentration (ppt)
    REAL(r_std),DIMENSION (npts,nslm), INTENT (in)        :: land_mcs         !! Mean of saturated volumetric water content for each layer in hydrol (m3/m3)
    REAL(r_std),DIMENSION (npts,nslm), INTENT (in)        :: stempdiag        !! Temperature profile on the levels in hydrol (K)
    REAL(r_std),DIMENSION(npts,nvm), INTENT (in)          :: resp_hetero_litter    !! Litter heterotrophic respiration per ground area 
                                                                                   !! @tex $(gC m^{-2} day^{-1})$ @endtex                                                                            
    REAL(r_std),DIMENSION(npts,nvm,nparts), INTENT (in)   :: resp_maint_part_radia !! Maintenance respiration of different plant parts per 
                                                                                   !! total ground area at Sechiba time step (gC m^{-2} dt_sechiba^{-1})

    !! 0.2 Output variables
    
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)         :: resp_hetero_soil !! Soil heterotrophic respiration \f$(gC m^{-2} (dt_sechiba one_day^{-1})^{-1})$\f

    !! 0.3 Modified variables
    
    REAL(r_std), DIMENSION(npts,ncarb,nvm), INTENT(inout) :: carbon             !! Soil carbon pools: active, slow, or passive, \f$(gC m^{2})$\f
    REAL(r_std), DIMENSION(npts,nvm,nbpools,nbpools), INTENT(inout) :: MatrixA  !! Matrix containing the fluxes between the carbon pools
                                                                                !! per sechiba time step 
                                                                                !! @tex $(gC.m^2.day^{-1})$ @endtex

    !! 0.4 Local variables
    REAL(r_std)                                           :: dt               !! Time step \f$(dt_sechiba one_day^{-1})$\f
    REAL(r_std), SAVE, DIMENSION(ncarb)                   :: carbon_tau       !! Residence time in carbon pools (days)
!$OMP THREADPRIVATE(carbon_tau)
    REAL(r_std), DIMENSION(npts,ncarb,ncarb)              :: frac_carb        !! Flux fractions between carbon pools 
                                                                              !! (second index=origin, third index=destination) 
                                                                              !! (unitless, 0-1)
    REAL(r_std), DIMENSION(npts,ncarb)                    :: frac_resp        !! Flux fractions from carbon pools to the atmosphere (respiration) (unitless, 0-1)
    REAL(r_std), DIMENSION(npts,ncarb,nelements)          :: fluxtot          !! Total flux out of carbon pools \f$(gC m^{2})$\f
    REAL(r_std), DIMENSION(npts,ncarb,ncarb,nelements)    :: flux             !! Fluxes between carbon pools \f$(gC m^{2})$\f
    CHARACTER(LEN=7), DIMENSION(ncarb)                    :: carbon_str       !! Name of the carbon pools for informative outputs (unitless)
    INTEGER(i_std)                                        :: k,kk,m,j,ij      !! Indices (unitless)
    REAL(r_std), DIMENSION(npts,nvm,ncarb)                :: decomp_rate_soilcarbon !! Decomposition rate of the soil carbon pools (s) 
    REAL(r_std), DIMENSION(npts,ncarb)                    :: tsoilpools       !! Diagnostic for soil carbon turnover rate by pool (1/s)
    REAL(r_std), DIMENSION(npts,nvm)                      :: Fcos_soil        !! Soil COS flux for oxic soils (pmol COS m^{-2} s^{-1})
    REAL(r_std), DIMENSION(npts)                          :: Fcos_soil_anox   !! Soil COS flux for anoxic soils (pmol COS m^{-2} s^{-1})
    REAL(r_std), DIMENSION(npts,nvm)                      :: Jsoil            !! Soil COS flux from Berry 2013 (pmol COS m^{-2} s^{-1})

!_ ================================================================================================================================

    !! printlev is the level of diagnostic information, 0 (none) to 4 (full)
    IF (printlev>=3) WRITE(numout,*) 'Entering soilcarbon' 

!! 1. Initializations
    dt = dt_sechiba/one_day
    !! 1.1 Get soil "constants"
    !! 1.1.1 Flux fractions between carbon pools: depend on clay content, recalculated each time
    ! From active pool: depends on clay content
    frac_carb(:,iactive,iactive) = zero
    frac_carb(:,iactive,ipassive) = frac_carb_ap
    frac_carb(:,iactive,islow) = un - (metabolic_ref_frac - active_to_pass_clay_frac*clay(:)) - frac_carb(:,iactive,ipassive)

    ! 1.1.1.2 from slow pool

    frac_carb(:,islow,islow) = zero
    frac_carb(:,islow,iactive) = frac_carb_sa
    frac_carb(:,islow,ipassive) = frac_carb_sp

    ! From passive pool
    frac_carb(:,ipassive,ipassive) = zero
    frac_carb(:,ipassive,iactive) = frac_carb_pa
    frac_carb(:,ipassive,islow) = frac_carb_ps

    IF ( firstcall_soilcarbon ) THEN

        !! 1.1.2 Residence times in carbon pools (days)
        carbon_tau(iactive) = carbon_tau_iactive * one_year       ! 1.5 years. This is same as CENTURY. But, in Parton et al. (1987), it's weighted by moisture and temperature dependences.
        carbon_tau(islow) = carbon_tau_islow * one_year          ! 25 years. This is same as CENTURY. But, in Parton et al. (1987), it's weighted by moisture and temperature dependences.
        carbon_tau(ipassive) = carbon_tau_ipassive * one_year       ! 1000 years. This is same as CENTURY. But, in Parton et al. (1987), it's weighted by moisture and temperature dependences.
        
        !! 1.2 Messages : display the residence times  
        carbon_str(iactive) = 'active'
        carbon_str(islow) = 'slow'
        carbon_str(ipassive) = 'passive'
        
        IF (printlev >= 2) THEN
           WRITE(numout,*) 'soilcarbon:'
           WRITE(numout,*) '   > minimal carbon residence time in carbon pools (d):'
           DO k = 1, ncarb ! Loop over carbon pools
              WRITE(numout,*) '(1, ::carbon_str(k)):',carbon_str(k),' : (1, ::carbon_tau(k)):',carbon_tau(k)
           ENDDO
           WRITE(numout,*) '   > flux fractions between carbon pools: depend on clay content'
        END IF
        firstcall_soilcarbon = .FALSE.
        
    ENDIF

    !! 1.3 Set soil respiration and decomposition rate to zero
    resp_hetero_soil(:,:) = zero
    decomp_rate_soilcarbon(:,:,:) = zero

!! 2. Update the carbon stocks with the different soil carbon input

    carbon(:,:,:) = carbon(:,:,:) + soilcarbon_input(:,:,:) * dt

!! 3. Fluxes between carbon reservoirs, and to the atmosphere (respiration) \n

    !! 3.1. Determine the respiration fraction : what's left after
    ! subtracting all the 'pool-to-pool' flux fractions
    ! Diagonal elements of frac_carb are zero
    !    VPP killer:
    !     frac_resp(:,:) = 1. - SUM( frac_carb(:,:,:), DIM=3 )
    frac_resp(:,:) = un - frac_carb(:,:,iactive) - frac_carb(:,:,islow) - &
         frac_carb(:,:,ipassive) 

    !! 3.2. Calculate fluxes

    DO m = 1, nvm ! Loop over # PFTs

      !! 3.2.1. Flux out of pools

      DO k = 1, ncarb ! Loop over carbon pools from which the flux comes
        
        ! Determine total flux out of pool
        ! S.L. Piao 2006/05/05 - for crop multiply tillage factor of decomposition
        ! Not crop
         IF ( natural(m) ) THEN
            fluxtot(:,k,icarbon) = dt/carbon_tau(k) * carbon(:,k,m) * &
                  control_moist(:,ibelow) * control_temp(:,ibelow)

            decomp_rate_soilcarbon(:,m,k)=dt/carbon_tau(k) * &
                      control_moist(:,ibelow) * control_temp(:,ibelow)
         ! C3 crop
          ELSEIF ( (.NOT. natural(m)) .AND. (.NOT. is_c4(m)) ) THEN
             fluxtot(:,k,icarbon) = dt/carbon_tau(k) * carbon(:,k,m) * &
                  control_moist(:,ibelow) * control_temp(:,ibelow) * flux_tot_coeff(1)
             decomp_rate_soilcarbon(:,m,k)=dt/carbon_tau(k) * &
                       control_moist(:,ibelow) * control_temp(:,ibelow)*flux_tot_coeff(1)
          ! C4 Crop   
          ELSEIF ( (.NOT. natural(m)) .AND. is_c4(m) ) THEN
             fluxtot(:,k,icarbon) = dt/carbon_tau(k) * carbon(:,k,m) * &
                  control_moist(:,ibelow) * control_temp(:,ibelow) * flux_tot_coeff(2)
             decomp_rate_soilcarbon(:,m,k)=dt/carbon_tau(k) * &
                       control_moist(:,ibelow) *control_temp(:,ibelow)*flux_tot_coeff(2)
        ENDIF
        ! END - S.L. Piao 2006/05/05 - for crop multiply tillage factor of decomposition

        ! Carbon flux from active pools depends on clay content
        IF ( k .EQ. iactive ) THEN
            fluxtot(:,k,icarbon) = fluxtot(:,k,icarbon) * ( un - flux_tot_coeff(3) * clay(:) )
            decomp_rate_soilcarbon(:,m,k)=decomp_rate_soilcarbon(:,m,k)* ( un - flux_tot_coeff(3) * clay(:) ) 
        ENDIF
        
        ! Update the loss in each carbon pool
        carbon(:,k,m) = carbon(:,k,m) - fluxtot(:,k,icarbon)
        
        ! Fluxes towards the other pools (k -> kk)
        DO kk = 1, ncarb ! Loop over the carbon pools where the flux goes
          flux(:,k,kk,icarbon) = frac_carb(:,k,kk) * fluxtot(:,k,icarbon)
        ENDDO
      ENDDO ! End of loop over carbon pools
      
      !! 3.2.2 respiration
      !BE CAREFUL: Here resp_hetero_soil is divided by dt to have a value which corresponds to
      ! the sechiba time step but then in stomate.f90 resp_hetero_soil is multiplied by dt.
      ! Perhaps it could be simplified. Moreover, we must totally adapt the routines to the dt_sechiba/one_day
      ! time step and avoid some constructions that could create bug during future developments.
      !
      !       VPP killer:
      !       resp_hetero_soil(:,m) = SUM( frac_resp(:,:) * fluxtot(:,:), DIM=2 ) / dt
      
      resp_hetero_soil(:,m) = &
         ( frac_resp(:,iactive) * fluxtot(:,iactive,icarbon) + &
         frac_resp(:,islow) * fluxtot(:,islow,icarbon) + &
         frac_resp(:,ipassive) * fluxtot(:,ipassive,icarbon)  ) / dt
      
      !! 3.2.3 add fluxes to active, slow, and passive pools
      !       VPP killer:
      !       carbon(:,:,m) = carbon(:,:,m) + SUM( flux(:,:,:), DIM=2 )
      
      DO k = 1, ncarb ! Loop over carbon pools
        carbon(:,k,m) = carbon(:,k,m) + &
           flux(:,iactive,k,icarbon) + flux(:,ipassive,k,icarbon) + flux(:,islow,k,icarbon)
      ENDDO ! Loop over carbon pools
      
    ENDDO ! End loop over PFTs
    
 !! 4. (Quasi-)Analytical Spin-up
    
    !! 4.1.1 Finish to fill MatrixA with fluxes between soil pools
    
    IF (spinup_analytic) THEN

       DO m = 2,nvm 

          ! flux leaving the active pool
          MatrixA(:,m,iactive_pool,iactive_pool) = moins_un * &
               dt/carbon_tau(iactive) * &
               control_moist(:,ibelow) * control_temp(:,ibelow) * &
               ( 1. - flux_tot_coeff(3) * clay(:)) 

          ! flux received by the active pool from the slow pool
          MatrixA(:,m,iactive_pool,islow_pool) =  frac_carb(:,islow,iactive)*dt/carbon_tau(islow) * &
               control_moist(:,ibelow) * control_temp(:,ibelow)

          ! flux received by the active pool from the passive pool
          MatrixA(:,m,iactive_pool,ipassive_pool) =  frac_carb(:,ipassive,iactive)*dt/carbon_tau(ipassive) * &
               control_moist(:,ibelow) * control_temp(:,ibelow) 

          ! flux received by the slow pool from the active pool
          MatrixA(:,m,islow_pool,iactive_pool) =  frac_carb(:,iactive,islow) *&
               dt/carbon_tau(iactive) * &
               control_moist(:,ibelow) * control_temp(:,ibelow) * &
               ( 1. - flux_tot_coeff(3) * clay(:) ) 

          ! flux leaving the slow pool
          MatrixA(:,m,islow_pool,islow_pool) = moins_un * &
               dt/carbon_tau(islow) * &
               control_moist(:,ibelow) * control_temp(:,ibelow)

          ! flux received by the passive pool from the active pool
          MatrixA(:,m,ipassive_pool,iactive_pool) =  frac_carb(:,iactive,ipassive)* &
               dt/carbon_tau(iactive) * &
               control_moist(:,ibelow) * control_temp(:,ibelow) *&
               ( 1. - flux_tot_coeff(3) * clay(:) )

          ! flux received by the passive pool from the slow pool
          MatrixA(:,m,ipassive_pool,islow_pool) =  frac_carb(:,islow,ipassive) * &
               dt/carbon_tau(islow) * &
               control_moist(:,ibelow) * control_temp(:,ibelow)

          ! flux leaving the passive pool
          MatrixA(:,m,ipassive_pool,ipassive_pool) =  moins_un * &
               dt/carbon_tau(ipassive) * &
               control_moist(:,ibelow) * control_temp(:,ibelow)      


          IF ( (.NOT. natural(m)) .AND. (.NOT. is_c4(m)) ) THEN ! C3crop

             ! flux leaving the active pool
             MatrixA(:,m,iactive_pool,iactive_pool) = MatrixA(:,m,iactive_pool,iactive_pool) * &
                  flux_tot_coeff(1)  

             ! flux received by the active pool from the slow pool
             MatrixA(:,m,iactive_pool,islow_pool)= MatrixA(:,m,iactive_pool,islow_pool) * &
                  flux_tot_coeff(1) 

             ! flux received by the active pool from the passive pool
             MatrixA(:,m,iactive_pool,ipassive_pool) = MatrixA(:,m,iactive_pool,ipassive_pool) * &
                  flux_tot_coeff(1)    

             ! flux received by the slow pool from the active pool
             MatrixA(:,m,islow_pool,iactive_pool) =  MatrixA(:,m,islow_pool,iactive_pool) * &
                  flux_tot_coeff(1) 

             ! flux leaving the slow pool
             MatrixA(:,m,islow_pool,islow_pool) = MatrixA(:,m,islow_pool,islow_pool) * &
                  flux_tot_coeff(1)     

             ! flux received by the passive pool from the active pool
             MatrixA(:,m,ipassive_pool,iactive_pool) = MatrixA(:,m,ipassive_pool,iactive_pool) * &
                  flux_tot_coeff(1)

             ! flux received by the passive pool from the slow pool
             MatrixA(:,m,ipassive_pool,islow_pool) = MatrixA(:,m,ipassive_pool,islow_pool) * &
                  flux_tot_coeff(1)

             ! flux leaving the passive pool
             MatrixA(:,m,ipassive_pool,ipassive_pool) =  MatrixA(:,m,ipassive_pool,ipassive_pool) *&
                  flux_tot_coeff(1)

          ENDIF ! (.NOT. natural(m)) .AND. (.NOT. is_c4(m)) 


          IF ( (.NOT. natural(m)) .AND. is_c4(m) ) THEN ! C4crop

             ! flux leaving the active pool
             MatrixA(:,m,iactive_pool,iactive_pool) = MatrixA(:,m,iactive_pool,iactive_pool) * &
                  flux_tot_coeff(2)  

            ! flux received by the active pool from the slow pool
             MatrixA(:,m,iactive_pool,islow_pool)= MatrixA(:,m,iactive_pool,islow_pool) * &
                  flux_tot_coeff(2) 

             ! flux received by the active pool from the passive pool
             MatrixA(:,m,iactive_pool,ipassive_pool) = MatrixA(:,m,iactive_pool,ipassive_pool) * &
                  flux_tot_coeff(2)    

             ! flux received by the slow pool from the active pool
             MatrixA(:,m,islow_pool,iactive_pool) =  MatrixA(:,m,islow_pool,iactive_pool) * &
                  flux_tot_coeff(2) 

             ! flux leaving the slow pool
             MatrixA(:,m,islow_pool,islow_pool) = MatrixA(:,m,islow_pool,islow_pool) * &
                  flux_tot_coeff(2)     

             ! flux received by the passive pool from the active pool
             MatrixA(:,m,ipassive_pool,iactive_pool) = MatrixA(:,m,ipassive_pool,iactive_pool) * &
                  flux_tot_coeff(2)

             ! flux received by the passive pool from the slow pool
             MatrixA(:,m,ipassive_pool,islow_pool) = MatrixA(:,m,ipassive_pool,islow_pool) * &
                  flux_tot_coeff(2)

             ! flux leaving the passive pool
             MatrixA(:,m,ipassive_pool,ipassive_pool) =  MatrixA(:,m,ipassive_pool,ipassive_pool) * &
                  flux_tot_coeff(2)

          ENDIF ! (.NOT. natural(m)) .AND. is_c4(m) 

          IF (printlev>=4) WRITE(numout,*)'Finish to fill MatrixA'

       ENDDO ! Loop over # PFTS


       ! 4.2 Add Identity for each submatrix(7,7) 

       DO j = 1,nbpools
          MatrixA(:,:,j,j) = MatrixA(:,:,j,j) + un 
       ENDDO

    ENDIF ! (spinup_analytic)


    ! Output diagnostics
    DO k = 1, ncarb ! Loop over carbon pools
       DO ij = 1, npts
          IF (SUM(decomp_rate_soilcarbon(ij,:,k)*veget_cov_max(ij,:)) > min_sechiba) THEN
             tsoilpools(ij,k) = 1./(SUM(decomp_rate_soilcarbon(ij,:,k)*veget_cov_max(ij,:))/dt_sechiba)
          ELSE
             tsoilpools(ij,k) = xios_default_val
          END IF
       END DO
    END DO
    CALL xios_orchidee_send_field("tSoilPools",tsoilpools)

    IF (printlev>=4) WRITE(numout,*) 'Leaving soilcarbon'
    
    CALL soilCOS_Ogee (npts, veget_cov_max, mcl_layh, pb, bulk, soil_pH, atmcos, &
      land_mcs, stempdiag, Fcos_soil, Fcos_soil_anox, nsoil_map)
    
    CALL soilCOS_Berry (npts, veget_cov_max, resp_hetero_soil, resp_hetero_litter, resp_maint_part_radia, mcl_layh, Jsoil)
    
  END SUBROUTINE soilcarbon


!! ================================================================================================================================
!!  SUBROUTINE   : soilCOS_Ogee
!!
!>\BRIEF        Computes COS fluxes between the soil and the atmosphere, essentially following Ogée et al., 2016.
!!
!! DESCRIPTION: The equations to calculate COS soil fluxes are found in Ogée et al., 2016, except the COS soil production term.
!!              The COS soil production term is described in Whelan et al., 2016. 
!!              The COS soil production term is not taken from Ogée et al., 2016 because we cannot account for the soil redox potential.
!!
!! The carbonic anhydrase enhancement factor (fCA) depends on biomes. 
!! Its values are given in Meredith et al., 2019 and adapted to ORCHIDEE PFTs
!! The alpha and beta parameters in the COS production term depend on biomes.
!! Their values are given in Whelan et al., 2016 and adapted to ORCHIDEE PFTs
!!
!! RECENT CHANGE(S): None
!! 
!! MAIN OUTPUTS VARIABLE(S): Fcos_soil, Fcos_soil_anox
!!
!! REFERENCE(S)   :
!! - Ogée, J.; Sauze, J.; Kesselmeier, J.; Genty, B.; Van Diest, H.; Launois, T.; Wingate, L. 
!!   A new mechanistic framework to predict OCS ﬂuxes from soils. 
!!   Biogeosciences 2016, 13, 22212240.
!!
!! - Whelan, M.E.; Hilton, T.W.; Berry, J.A.; Berkelhammer, M.; Desai, A.R.; Campbell, J.E. 
!!   Carbonyl sulﬁde exchange in soils for better estimates of ecosystem carbon uptake. 
!!   Atmos. Chem. Phys. 2016, 16, 37113726.
!! 
!! - Laura K. Meredith, Kristin Boye, Connor Youngerman, Mary Whelan, Jérôme Ogée, Joana Sauze and Lisa Wingate.
!!   Coupled Biological and Abiotic Mechanisms Driving Carbonyl Sulﬁde Production in Soils.
!!   Soil Syst. 2018, 2, 37; doi:10.3390/soilsystems2030037.
!!
!!
!_ ================================================================================================================================
  SUBROUTINE soilCOS_Ogee (npts, veget_cov_max, mcl_layh, pb, bulk, soil_pH, atmcos, &
      land_mcs, stempdiag, Fcos_soil, Fcos_soil_anox, nsoil_map)

!! 0. Variable and parameter declaration

    !! 0.1 Input variables
    
    INTEGER(i_std), INTENT(in)                            :: npts             !! Domain size (unitless)
    REAL(r_std), DIMENSION(npts), INTENT(in)              :: nsoil_map         !! Dominant soil type between oxic and anoxic (0-4, unitless)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)          :: veget_cov_max    !! Fractional coverage: maximum share of the pixel taken by a pft 
    REAL(r_std),DIMENSION (npts,nslm), INTENT (in)        :: mcl_layh         !! Volumetric moisture content for each layer in hydrol (liquid) (m3/m3) 
    REAL(r_std),DIMENSION (npts), INTENT (in)             :: pb               !! Surface pressure (hPa)
    REAL(r_std),DIMENSION(npts),INTENT(in)                :: bulk             !! Bulk density (kg/m**3)
    REAL(r_std),DIMENSION (npts), INTENT (in)             :: soil_pH          !! Soil pH
    REAL(r_std),DIMENSION (:,:), INTENT (in)              :: atmcos           !! Atmospheric COS concentration (ppt)
    REAL(r_std),DIMENSION (npts,nslm), INTENT (in)        :: land_mcs         !! Mean of saturated volumetric water content for each layer in hydrol (m3/m3)
    REAL(r_std),DIMENSION (npts,nslm), INTENT (in)        :: stempdiag        !! Temperature profile on the levels in hydrol (K)

    !! 0.2 Output variables
    
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)         :: Fcos_soil        !! Soil COS flux for oxic soils (pmol COS m^{-2} s^{-1})
    REAL(r_std), DIMENSION(npts), INTENT(out)             :: Fcos_soil_anox   !! Soil COS flux for anoxic soils (pmol COS m^{-2} s^{-1})

    !! 0.3 Modified variables

    !! 0.4 Local variables

    INTEGER(i_std)                                        :: m,jsl,n,ij,j3h   !! Indices (unitless) 
    INTEGER(i_std)                                        :: ox,anox,iox,ianox!! counter/indices for oxic or anoxic soil types (unitless) 
    INTEGER(i_std), DIMENSION(npts)                       :: index_ox,index_anox     !! indices for oxic or anoxic soil types (unitless)
    INTEGER(i_std), DIMENSION(13)                         :: pf               !! Indices of PFTs other than crops (unitless)
    REAL(r_std)                                           :: dt               !! Time step \f$(dt_sechiba one_day^{-1})$\f
    REAL(r_std), DIMENSION(npts)                          :: w_first_layers   !! Weighted average of the volumetric water content over the first soil layers 
                                                                              !! @tex (m^{3} water m^{-3} soil) @endtex
    REAL(r_std), DIMENSION(npts)                          :: T_first_layers   !! Weighted average temperature over the first soil layers (K)
    REAL(r_std), DIMENSION(npts)                          :: poros_first_layers   !! Weighted average of the mean saturated volumetric water content over the first soil layers 
                                                                              !! @tex (m^{3} m^{-3}) @endtex
    REAL(r_std), DIMENSION(npts,nvm)                      :: P                !! OCS production term for oxic soils (mol/m3/s)
    REAL(r_std), DIMENSION(npts)                          :: P_anox           !! OCS production term for anoxic soils (pmol/m3/s)
    REAL(r_std), DIMENSION(npts)                          :: xCA              !! Ratio kmax/Km temperature dependency
    REAL(r_std), DIMENSION(npts,nvm)                      :: k                !! Total OCS hydrolysis rate (s^{-1})
    REAL(r_std), DIMENSION(npts)                          :: kH               !! Henry's law constant (mol/m3/Pa)
    REAL(r_std), DIMENSION(npts)                          :: B                !! COS solubility (m3/m3)
    REAL(r_std), DIMENSION(npts,nvm)                      :: tau_a            !! Tortuosity factor for gaseous diffusion (unitless)
    REAL(r_std), DIMENSION(npts)                          :: tau_l            !! Tortuosity factor for liquid diffusion (unitless)
    REAL(r_std), DIMENSION(npts)                          :: D0_a             !! Binary diffusivity of COS in the free air (m2 air/s)
    REAL(r_std), DIMENSION(npts,nvm)                      :: Deff_a           !! Effective diffusivity of gaseous COS through the soil matrix (m3 air/m soil/s)
    REAL(r_std), DIMENSION(npts)                          :: D0_l             !! Binary diffusivity of COS in the free water (m2 water/s)
    REAL(r_std), DIMENSION(npts)                          :: Deff_l           !! Effective diffusivity of dissolved COS through the soil matrix (m3 water/m soil/s)
    REAL(r_std), DIMENSION(npts,nvm)                      :: D                !! Total effective COS diffusivity in soil (in air and in water) (m2/s)
    REAL(r_std), DIMENSION(npts,nvm)                      :: z1               !! Caracteristic depth (m)
                                                                             
!_ ================================================================================================================================

!! 1. Initializations
    Fcos_soil(:,:) = zero
    Fcos_soil_anox(:) = zero
    w_first_layers(:) = zero
    T_first_layers(:) = zero
    poros_first_layers(:) = zero
    P(:,:) = zero
    P_anox(:) = zero
    xCA(:) = zero
    k(:,:) = zero
    kH(:) = zero
    B(:) = zero
    tau_a(:,:) = zero
    tau_l(:) = zero
    D0_a(:) = zero
    Deff_a(:,:) = zero
    D0_l(:) = zero
    Deff_l(:) = zero
    D(:,:) = zero
    z1(:,:) = zero
    ox = zero
    anox = zero
    index_ox(:) = zero 		
    index_anox(:) = zero	

!! 2. Temperature, volumetric moisture content and porosity in the first soil layers
    
    ! 2.1. Weighted average temperature in the first soil layers
    DO jsl=1,6 ! Loop over soil layers
        T_first_layers(:) = T_first_layers(:) + stempdiag(:,jsl) * dlh(jsl)
    ENDDO ! End loop over soil layers
    T_first_layers(:) = T_first_layers(:) / zlh(6)
    
    ! 2.2. Weighted average volumetric moisture content in the first soil layers
    DO jsl=1,6 ! Loop over soil layers
        w_first_layers(:) = w_first_layers(:) + mcl_layh(:,jsl) * dlh(jsl)
    ENDDO ! End loop over soil layers
    w_first_layers(:) = w_first_layers(:) / zlh(6)
    
    ! Soil porosity is constant in the soil column for each grid cell
    ! We use the value in the first layer land_mcs(:,1) 
    poros_first_layers(:) = land_mcs(:,1)
    
!! 3. COS consumption

    ! 3.1. Hydrolysis by carbonic anhydrase enzyme 
    ! in soil microorganisms at a catalysed rate kcat
    
    ! Ratio kmax/Km temperature dependency
    ! Ogée et al. (2016) equation 10.a
    xCA(:) = exp( -1. * delta_Ha / ( RR * T_first_layers(:) ) ) / &
          ( 1. + exp( ( -1. * delta_Hd / ( RR * T_first_layers(:) ) ) + ( delta_Sd / RR ) ))

    ! 3.2. Total COS hydrolysis rate k
    ! Ogée et al. (2016) equation 11.b
    DO m = 1, nvm ! Loop over PFTs
        k(:,m) = fCA(m) * kuncat * xCA(:) / xCA_T25
    ENDDO ! End loop over PFTs
    
!! 4. Solubility
 
    ! 4.1. Henry's law constant kH
    ! Ogée et al. (2016) equation page 3
    kH(:) = 0.00021 * exp( 24900. * ( ( 1 / T_first_layers(:) ) - ( 1. / Tref ) ) / RR )
    
    ! 4.2. COS solubility B
    ! Ogée et al. (2016) equation page 3
    B(:) = kH(:) * RR * T_first_layers(:)
    
!! 5. Tortuosity factors

    ! 5.1. Tortuosity factors for gaseous diffusion
        
        ! 5.1.1. Gaseous diffusion in repacked soils tau_a_r
        ! (agricultural soils)
        ! Ogée at al. (2016) table 1 Mol03r
    DO m = 12, 13 ! Loop over crops
        tau_a(:,m) = ( ( poros_first_layers(:) - w_first_layers(:) )**( 3. / 2. ) ) / &
              poros_first_layers(:)
    ENDDO ! End loop over crops
    
        ! 5.1.2. Gaseous diffusion in undisturbed soils tau_a_u
        ! Ogée et al. (2016) table 1 Deepa11
    ! Indexes of PFTs other than crops
    pf = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 14, 15] 
    DO m = 1, 13 ! loop over PFTs other than crops
        n = pf(m)
        tau_a(:,n) = ( ( 0.2 * ( ( ( poros_first_layers(:) - w_first_layers(:) ) / &
              poros_first_layers(:) )**2. ) ) + 0.004 ) / poros_first_layers(:)
    ENDDO ! End loop over PFTs other than crops

    ! 5.2. Tortuosity factor for liquid diffusion tau_l
    ! Ogée et al. (2016) table 1 MQ61
    tau_l(:) = ( w_first_layers(:)**( 7. / 3. ) ) / ( poros_first_layers(:)**2. )
    
!! 6. Diffusivity

    ! 6.1. COS diffusivity in the air
    
        ! 6.1.1. Binary diffusivity of COS in the free air D0_a
        ! Ogée et al. (2016) equation page 4
    D0_a(:) = D0_a_T0p0 * ( ( T_first_layers(:) / Tref )**1.5 ) * ( p0 / ( pb(:) * 100. ) )
        
        ! 6.1.2. Effective diffusivity of gaseous COS through the soil matrix Deff_a
        ! Ogée et al. (2016) equation page 4
    DO m = 1, nvm ! Loop over PFTs
        Deff_a(:,m) = tau_a(:,m) * ( poros_first_layers(:) - w_first_layers(:) ) * D0_a(:)
    ENDDO ! Loop over PFTs
    
    ! 6.2. COS diffusivity in the water
    
        ! 6.2.1. Binary diffusivity of COS in the free water D0_l
        ! Ogée et al. (2016) equation page 4
    D0_l(:) = D0_l_T1 * ( ( T_first_layers(:) / 216. ) - 1. )**2. 
    
        ! 6.1.2. Effective diffusivity of dissolved COS through the soil matrix Deff_l
        ! Ogée et al. (2016) equation page 4
    Deff_l(:) = tau_l(:) * w_first_layers(:) * D0_l(:)  
    
    ! 6.3. COS diffusivity D
    ! Li (2018) table 1
    DO m = 1, nvm ! Loop over PFTs
        D(:,m) = Deff_a(:,m) + ( Deff_l(:) * B(:) )
    ENDDO ! End loop over PFTs
    
!! 7. Depth z1
! Ogée et al. (2016) equation 17
    DO m = 1, nvm ! Loop over PFTs
        z1(:,m) = sqrt( D(:,m) / ( k(:,m) * B(:) * w_first_layers(:) ) )
    ENDDO ! End loop over PFTs

!! 8. Differenciate between oxic and anoxic soils
! Changes the production term

    ! 8.1. Find index for oxic or anoxic soil types
    ! Distinction between oxic and anoxic soil is made by using the map 
    ! from Tootchi et al. (2019)
    DO ij = 1,npts 
        IF ((nsoil_map(ij)) .EQ. 0) THEN
            ox = ox + 1
            index_ox(ox) = ij
        ELSEIF ((nsoil_map(ij)) .GT. 0) THEN
            anox = anox + 1
            index_anox(anox) = ij
        ENDIF
    ENDDO

    ! 8.2. COS production term P for oxic soils
    ! Whelan et al. (2016) equation 4
    ! There is an error in the paper
    ! Correct equation: P = exp(alpha + beta*T(°C)) in pmol/g/min
    ! P is converted from (pmol/g/min) to (mol/m3/s)
    ! P = exp(alpha + beta*T(°C)) * bulk(g/m3) / 60 / 10**12 in mol/m3/s
    ! bulk density is converted from kg/m3 to g/m3
    ! The production term for bare soil is zero
	DO m = 2, nvm ! Loop over PFTs
		DO iox=1,ox	
			P(index_ox(iox),m) = ( exp( alpha_Wh(m) + ( beta_Wh(m) * ( T_first_layers(index_ox(iox)) - 273.15) ) ) * &	
              ( bulk(index_ox(iox)) * 1000. ) / 60. ) / 10.**12.
		ENDDO 		
	ENDDO ! End loop over PFTs
    
    ! 8.3. COS production term P_anox for anoxic soils
    ! Ogée et al. (2016) equation 12a
    ! This production term does not depend on the PFT but only on soil temperature
    ! Soil redox potential is estimated at -200 mV for anoxic soils by lack of more information on its dynamic
    DO ianox=1,anox	
		P_anox(index_anox(ianox)) = ( Pref / zlh(6) ) * ( cos_Q10 ** ( ( T_first_layers(index_anox(ianox)) - Tref ) / 10 ) )	
	ENDDO 		

!! 9. Net gas exchange between the soil and the atmosphere Fcos_soil
! Ogée et al. (2016) equation 17
    IF (read_atmcos) THEN
        DO m = 1, nvm ! Loop over PFTs
            ! Oxic soils
			DO iox=1,ox	
                DO j3h = 0,7
                   IF (sec_start .EQ. j3h*3*one_hour .OR. sec_start .GE. j3h*3*one_hour .AND. sec_start .LE. (j3h+1)*3*one_hour) THEN 
                       Fcos_soil(index_ox(iox),m) = -1. * sqrt( k(index_ox(iox),m) * B(index_ox(iox)) * w_first_layers(index_ox(iox)) * &	
				    	D(index_ox(iox),m) ) * ( ( ( p0 * atmcos(index_ox(iox),(julian_diff-1)*8+j3h+1) / 10.**12. ) / ( RR * Tref ) ) - &	
					    ( ( ( ( z1(index_ox(iox),m)**2. ) * P(index_ox(iox),m) ) / D(index_ox(iox),m) ) * &		
					    ( 1. - exp( ( -1. * zlh(6) ) / z1(index_ox(iox),m) ) ) ) ) * 10.**12.	
		           ENDIF 
		        ENDDO 
			ENDDO  	
        ENDDO ! End loop over PFTs
    ELSE
        DO m = 1, nvm ! Loop over PFTs
            ! Oxic soils
			DO iox=1,ox	
				Fcos_soil(index_ox(iox),m) = -1. * sqrt( k(index_ox(iox),m) * B(index_ox(iox)) * w_first_layers(index_ox(iox)) * &	
					D(index_ox(iox),m) ) * ( ( ( p0 * atmcos(1,1) / 10.**12. ) / ( RR * Tref ) ) - &										
					( ( ( ( z1(index_ox(iox),m)**2. ) * P(index_ox(iox),m) ) / D(index_ox(iox),m) ) * &							
					( 1. - exp( ( -1. * zlh(6) ) / z1(index_ox(iox),m) ) ) ) ) * 10.**12. 										
 			ENDDO 		
        ENDDO ! End loop over PFTs
        ! Anoxic soil
    ENDIF
    DO ianox=1,anox	
		Fcos_soil_anox(index_anox(ianox)) = P_anox(index_anox(ianox)) * zlh(6)	
	ENDDO 
      
    CALL xios_orchidee_send_field("Fcos_soil",Fcos_soil)
    CALL xios_orchidee_send_field("Fcos_soil_anox",Fcos_soil_anox)
    CALL xios_orchidee_send_field("kH",kH)
    CALL xios_orchidee_send_field("B",B)
    CALL xios_orchidee_send_field("Deff_a", Deff_a)
    CALL xios_orchidee_send_field("k",k)
    CALL xios_orchidee_send_field("Deff_l", Deff_l)
    CALL xios_orchidee_send_field("D", D)
    CALL xios_orchidee_send_field("T_first_layers", T_first_layers)
    CALL xios_orchidee_send_field("w_first_layers", w_first_layers)
    CALL xios_orchidee_send_field("poros_first_layers", poros_first_layers)
    CALL xios_orchidee_send_field("P", P)
    CALL xios_orchidee_send_field("P_anox", P_anox)
    CALL xios_orchidee_send_field("bulk", bulk)
    CALL xios_orchidee_send_field("nsoil_map",nsoil_map)

  END SUBROUTINE soilCOS_Ogee
  

!! ================================================================================================================================
!!  SUBROUTINE   : soilCOS_Berry
!!
!>\BRIEF        Computes COS fluxes between the soil and the atmosphere, essentially following Berry et al., 2013.
!!
!! DESCRIPTION: Only estimate COS soil uptake and not COS soil production
!!
!_ ================================================================================================================================
  SUBROUTINE soilCOS_Berry (npts, veget_cov_max, resp_hetero_soil, resp_hetero_litter, resp_maint_part_radia, mcl_layh, Jsoil)

!! 0. Variable and parameter declaration

    !! 0.1 Input variables
    
    INTEGER(i_std), INTENT(in)                            :: npts             !! Domain size (unitless)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)          :: veget_cov_max    !! Fractional coverage: maximum share of the pixel taken by a pft 
    REAL(r_std),DIMENSION(npts,nvm), INTENT (in)          :: resp_hetero_soil !! Soil heterotrophic respiration 
                                                                              !! (gC m^{-2} (dt_sechiba one_day^{-1})^{-1})
    REAL(r_std),DIMENSION(npts,nvm), INTENT (in)          :: resp_hetero_litter    !! Litter heterotrophic respiration per ground area 
                                                                                   !! @tex $(gC m^{-2} day^{-1})$ @endtex                                                                            
    REAL(r_std),DIMENSION(npts,nvm,nparts), INTENT (in)   :: resp_maint_part_radia !! Maintenance respiration of different plant parts per 
                                                                                   !! total ground area at Sechiba time step (gC m^{-2} dt_sechiba^{-1})
    REAL(r_std),DIMENSION (npts,nslm), INTENT (in)        :: mcl_layh         !! Volumetric moisture content for each layer in hydrol (liquid) m3/m3 

    !! 0.2 Output variables
    
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)         :: Jsoil            !! Soil COS flux (pmol COS m^{-2} s^{-1})

    !! 0.3 Modified variables

    !! 0.4 Local variables
    
    INTEGER(i_std)                                        :: m,jsl            !! Indices (unitless)
    REAL(r_std)                                           :: dt               !! Time step \f$(dt_sechiba one_day^{-1})$\f
    REAL(r_std), DIMENSION(npts,nvm)                      :: resp_total_soil  !! Total soil respiration per PFT
                                                                              !! @tex (umol CO2 m^{-2} s^{-1}) @endtex
                                                                              
!_ ================================================================================================================================

!! 1. Initializations
    Jsoil(:,:) = zero
    resp_total_soil(:,:) = zero

!! 2. Sum of soil heterotrophic and autotrophic respiration per PFT
    DO m = 1, nvm ! Loop over PFTs
        resp_total_soil(:,m) = ( ( resp_hetero_soil(:,m) * (dt_sechiba / one_day) ) + resp_hetero_litter(:,m) + &
        resp_maint_part_radia(:,m,isapbelow) + resp_maint_part_radia(:,m,iroot) ) * 10.**6 / ( 12. * dt_sechiba ) 
    ENDDO ! End loop over PFTs
    
!! 4. Soil COS uptake (pmol COS m^{-2} s^{-1})
    DO m = 1, nvm ! Loop over PFTs
        Jsoil(:,m) = - ksoil * resp_total_soil(:,m)
    ENDDO ! End loop over PFTs
    
    CALL xios_orchidee_send_field("Jsoil",Jsoil)
    CALL xios_orchidee_send_field("resp_total_soil",resp_total_soil)

  END SUBROUTINE soilCOS_Berry

END MODULE stomate_soilcarbon
